@@ -188,10 +188,10 @@ mpi_read (iobuf_t inp, unsigned int *ret_nread, int secure)
 }
 
 
-/* Read an external representation of an SOS and return the opaque MPI
-   with GCRYMPI_FLAG_USER2.  The external format is a 16-bit unsigned
-   value stored in network byte order giving information for the
-   following octets.
+/* Read an external representation (which is possibly an SOS) and
+   return the MPI.  The external format is a 16-bit unsigned value
+   stored in network byte order giving information for the following
+   octets.
 
    The caller must set *RET_NREAD to the maximum number of bytes to
    read from the pipeline INP.  This function sets *RET_NREAD to be
@@ -200,11 +200,12 @@ mpi_read (iobuf_t inp, unsigned int *ret_nread, int secure)
    If SECURE is true, the integer is stored in secure memory
    (allocated using gcry_xmalloc_secure).  */
 static gcry_mpi_t
-sos_read (iobuf_t inp, unsigned int *ret_nread, int secure)
+mpi_read_detect_0_removal (iobuf_t inp, unsigned int *ret_nread, int secure,
+                           u16 *r_csum_tweak)
 {
   int c, c1, c2, i;
   unsigned int nmax = *ret_nread;
-  unsigned int nbits, nbytes;
+  unsigned int nbits, nbits1, nbytes;
   size_t nread = 0;
   gcry_mpi_t a = NULL;
   byte *buf = NULL;
@@ -229,8 +230,10 @@ sos_read (iobuf_t inp, unsigned int *ret_nread, int secure)
     }
 
   nbytes = (nbits + 7) / 8;
-  buf = secure ? gcry_xmalloc_secure (nbytes) : gcry_xmalloc (nbytes);
+  buf = secure ? gcry_xmalloc_secure (nbytes + 2) : gcry_xmalloc (nbytes + 2);
   p = buf;
+  p[0] = c1;
+  p[1] = c2;
   for (i = 0; i < nbytes; i++)
     {
       if (nread == nmax)
@@ -240,13 +243,29 @@ sos_read (iobuf_t inp, unsigned int *ret_nread, int secure)
       if (c == -1)
         goto leave;
 
-      p[i] = c;
+      p[i + 2] = c;
+
       nread ++;
     }
 
-  a = gcry_mpi_set_opaque (NULL, buf, nbits);
-  gcry_mpi_set_flag (a, GCRYMPI_FLAG_USER2);
+  if (gcry_mpi_scan (&a, GCRYMPI_FMT_PGP, buf, nread, &nread))
+    a = NULL;
+
+  /* Possibly, it has leading zeros.  */
+  if (a)
+    {
+      nbits1 = gcry_mpi_get_nbits (a);
+      if (nbits > nbits1)
+        {
+          *r_csum_tweak -= (nbits >> 8);
+          *r_csum_tweak -= (nbits & 0xff);
+          *r_csum_tweak += (nbits1 >> 8);
+          *r_csum_tweak += (nbits1 & 0xff);
+        }
+    }
+
   *ret_nread = nread;
+  gcry_free(buf);
   return a;
 
  overflow:
@@ -2290,8 +2309,10 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
 	}
       else
 	{
-	  sig->data[0] =
-	    gcry_mpi_set_opaque (NULL, read_rest (inp, pktlen), pktlen * 8);
+          void *tmpp;
+
+          tmpp = read_rest (inp, pktlen);
+	  sig->data[0] = gcry_mpi_set_opaque (NULL, tmpp, tmpp? pktlen * 8 : 0);
 	  pktlen = 0;
 	}
     }
@@ -2499,8 +2520,10 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
   if (!npkey)
     {
       /* Unknown algorithm - put data into an opaque MPI.  */
-      pk->pkey[0] = gcry_mpi_set_opaque (NULL,
-                                         read_rest (inp, pktlen), pktlen * 8);
+      void *tmpp = read_rest (inp, pktlen);
+      /* Current gcry_mpi_cmp does not handle a (NULL,n>0) nicely and
+       * thus we avoid to create such an MPI.  */
+      pk->pkey[0] = gcry_mpi_set_opaque (NULL, tmpp, tmpp? pktlen * 8 : 0);
       pktlen = 0;
       goto leave;
     }
@@ -2764,6 +2787,8 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
 	}
       else if (ski->is_protected)
 	{
+          void *tmpp;
+
 	  if (pktlen < 2) /* At least two bytes for the length.  */
 	    {
               err = gpg_error (GPG_ERR_INV_PACKET);
@@ -2773,9 +2798,10 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
 	  /* Ugly: The length is encrypted too, so we read all stuff
 	   * up to the end of the packet into the first SKEY
 	   * element.  */
+
+          tmpp = read_rest (inp, pktlen);
 	  pk->pkey[npkey] = gcry_mpi_set_opaque (NULL,
-						 read_rest (inp, pktlen),
-						 pktlen * 8);
+						 tmpp, tmpp? pktlen * 8 : 0);
           /* Mark that MPI as protected - we need this information for
              importing a key.  The OPAQUE flag can't be used because
              we also store public EdDSA values in opaque MPIs.  */
@@ -2787,6 +2813,8 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
 	}
       else
 	{
+          u16 csum_tweak = 0;
+
           /* Not encrypted.  */
 	  for (i = npkey; i < nskey; i++)
 	    {
@@ -2799,7 +2827,8 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
                 }
               n = pktlen;
               if (algorithm == PUBKEY_ALGO_EDDSA)
-                pk->pkey[i] = sos_read (inp, &n, 0);
+                pk->pkey[i] = mpi_read_detect_0_removal (inp, &n, 0,
+                                                         &csum_tweak);
               else
                 pk->pkey[i] = mpi_read (inp, &n, 0);
               pktlen -= n;
@@ -2822,6 +2851,7 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
 	      goto leave;
 	    }
 	  ski->csum = read_16 (inp);
+          ski->csum += csum_tweak;
 	  pktlen -= 2;
 	  if (list_mode)
             es_fprintf (listfp, "\tchecksum: %04hx\n", ski->csum);
