@@ -30,6 +30,7 @@
 
 
 /* We keep all global options in the structure OPT.  */
+EXTERN_UNLESS_MAIN_MODULE
 struct
 {
   int verbose;
@@ -37,6 +38,7 @@ struct
   int quiet;
   int use_sendmail;
   int with_colons;
+  int no_autostart;
   const char *output;
   const char *gpg_program;
   const char *directory;
@@ -77,6 +79,7 @@ struct uidinfo_list_s
   struct uidinfo_list_s *next;
   time_t created; /* Time the userid was created.  */
   char *mbox;  /* NULL or the malloced mailbox from UID.  */
+  unsigned int flags;  /* These flags are cleared on creation.  */
   char uid[1];
 };
 typedef struct uidinfo_list_s *uidinfo_list_t;
@@ -98,6 +101,16 @@ gpg_error_t wks_parse_policy (policy_flags_t flags, estream_t stream,
                               int ignore_unknown);
 void wks_free_policy (policy_flags_t policy);
 
+gpg_error_t wks_fname_from_userid (const char *userid, int hash_only,
+                                   char **r_fname, char **r_addrspec);
+gpg_error_t wks_compute_hu_fname (char **r_fname, const char *addrspec);
+gpg_error_t wks_install_key_core (estream_t key, const char *addrspec);
+gpg_error_t wks_cmd_install_key (const char *fname, const char *userid);
+gpg_error_t wks_cmd_remove_key (const char *userid);
+gpg_error_t wks_cmd_print_wkd_hash (const char *userid);
+gpg_error_t wks_cmd_print_wkd_url (const char *userid);
+
+
 /*-- wks-receive.c --*/
 
 /* Flag values for the receive callback.  */
